function [dev,b,Mu,Pco,Pmal,se,Fm,lky] = emglom(Y,nst,ncl,lev,Z,rec,mit,tol,dev,b)

% -      [dev,b,Mu,Pco,Pmal,se,Fm,lky] = emglom(Y,nst,ncl,lev,Z,<rec>,<mit>,<tol>,<dev>,<b>)
% -
% - fits GLLM model with latent classes using the EM algorithm
% -
% - inputs:
% - y    : incomplete data vector
% - nst  : numbero of stratas
% - ncl  : numbero of classes
% - lev  : list of the levels of the variables
% - Z    : design matrix
% - rec  : 1 for capture-recapture problem, 0 otherwise (deafualt)
%          the first entry of the table is missing
% - dev  : deviance to be reached (put 0 to be not considered)
% - mit  : maximum number of iteration 
% - tol  : tollerance
% - b    : starting values of b
% - 
% - outputs:
% - dev  : deviance reached
% - b    : parameters estimation vector (column)
% - Mu   : estimated frequencies (obs x lat)
% - Pco  : prob (obs | lat)
% - Pmal : (marginal lat) prob

% - adjust inputs
  if nargin < 6, rec = 0; end
  if nargin < 7, mit = 2000; end
  if nargin < 8, tol = 0.0001; dev=0; end
  if nargin < 9, dev = 0; end
  if nargin >= 9 & dev>0, tol=0; end

% - preliminaries
  yl = size(Y,1);  yt=sum(Y)';
  J = length(lev);
  Y = Y+10^(-8)*(Y==0);
  m = prod(lev); 
  n = m*lev;
  p = size(Z,2);                               % - number of parameters
  Lp = kron(eye(nst),kron (ones(1,ncl), eye(m)));
  L = kron(eye(nst),kron (ones(1,ncl), eye(m-1)));
  Np = kron(eye(nst),kron(ones(1,ncl),ones(1,m)));
  N = kron(eye(nst),kron(ones(1,ncl),ones(1,m-1)));
  [a,M,C] = glom(2*ones(1,J),ones(1,J),'f');

% - initial estimates
  if nargin < 10,
      b = stglom(Y,nst,ncl,lev,Z,a,M,C,rec);
%      b = zeros(size(Z,2),1);
    else
      if length(b) ~= p,   
        error('check number of parameters');
      end
      b = shape(b,p);
  end

% - iterate
  dis = 1; it = 0; dev = 10^10; devo = dev + 1;
  eta = Z * b;
  [la,pi,p,lap,pip,pp] = prob(nst,ncl,J,eta,a,M,C);
  if rec == 0
      mu = diag(kron(sum(Y)',ones(ncl*m,1)))*pp;
    else
      mu = diag(kron(sum(Y)'./(N*p),ones(ncl*m,1)))*pp;
  end
% iterate
  while (dis > tol) & (it < mit),% & ((devo > dev)|it<10), 
    it = it+1; out = mu; devo = dev;
    bs = b;
    b = b + emsglom(Y,nst,ncl,J,Z,L,N,b,rec,a,M,C);
    eta = Z*b;
    [la,pi,p,lap,pip,pp] = prob(nst,ncl,J,eta,a,M,C);
    if rec == 0,
      mu = diag(kron(sum(Y)',ones(ncl*m,1)))*pp;
    else
      mu = diag(kron(sum(Y)'./(N*p),ones(ncl*m,1)))*pp;
    end
    l = Lp * mu;
    l = l+10^(-8)*(l < 10^(-8));  
    y = reshape(Y,size(Y,1)*size(Y,2),1);
    if rec == 0,
      dev = 2*sum(y.*log(y./l ));
      La = reshape(l,length(l)/nst,nst);
    else
      La = reshape(l,length(l)/nst,nst); Lap = La;
      La(1,:) = [];
      l = reshape(La,size(La,1)*size(La,2),1);
      dev = 2*sum(y.*log(y./l ))-2*sum(y-l);
    end
    t1 = mean(abs(mu-out)./mu);       % - varation of mu
    t2 = devo-dev;             % - varation of deviance
    dis = max(t1,t2); 
    if floor(it/10) == it/10,
      Dev = 2*sum(Y.*log(Y./La ));
      disp([it Dev dev t1 t2]);
    end
%    keyboard
  end
  Dev = 2*sum(Y.*log(Y./La ));
  disp([it Dev dev t1 t2]);
  
% - computes standard errors, expected frequencies and probablities
  Mu = shape(mu,m);
  Pint = Mu/sum(sum(Mu));
  Pmal = sum(Pint);
  Pco = Pint*inv(diag(Pmal));
  [lky,gv,Fm] = lkyglom(nst,ncl,J,Y,Z,b,a,M,C,rec);
  se = sqrt(diag(inv(Fm)));
% - Displaies output
  disp('Estimates, s.e.  and  z');
  disp([b se b./se]);
  disp(' ');
  disp('Deviance');
  disp(dev);
  if rec==1,
    disp('Total pop. esimate');
    disp(sum(Lap));
  end