function [la,pi,p,lap,pip,pp,D,Dp] = prob(nst,ncl,J,eta,a,M,C,sw)

% -      [la,pi,p,lap,pip,pp,D,Dp] = prob(nst,ncl,J,eta,a,M,C,sw)
% -
% - computes probabilities on the basis of the 
% - GLMM conditional parametrization
% -
% - inputs:
% - Z    : design matrix
% - b    : vector of the parameters
% - nst  : number of stratas
% - 
% - outputs:
% - pj   : variation of parameters estimation vector (column)
% - pm   : variation of parameters estimation vector (column)
% - pc   : variation of parameters estimation vector (column)
% - sw   : 2 computed also the first derivative matrix of
%          p with respect eta' (D) and of pp with respect eta' (Dp)
 
  if nargin<8, sw =1; end
  t = 2^J-1;
  sc = nst*(ncl-1); 
  eta0 = eta(1:sc);
  Pi = []; Pip = [];
  for s = 1:nst,
    i = (s-1)*(ncl-1)+(1:ncl-1);
    v = eta(i);
    p = exp(v)/(1+sum(exp(v)));
    Pi = [Pi p];
    Pip = [Pip [1-sum(p); p]];
  end
  Pc = []; Pj = [];
  Pcp = []; Pjp = [];
  for s = 1:nst,
    pc = []; pj = [];
    pcp = []; pjp = [];
    for c = 1:ncl,
      i = sc + (s-1)*ncl*t + (c-1)*t;
      i = i+(1:t);
      v = eta(i); v = glinv(a,M,C,v);
      vp = [1-sum(v); v];
      pc = [pc; v];
      pj = [pj; v*Pip(c,s)];
      pcp = [pcp; vp];
      pjp = [pjp; vp*Pip(c,s)];
    end
    Pcp = [Pcp pcp]; Pjp = [Pjp pjp];
    Pc = [Pc pc]; Pj = [Pj pj];
  end  
  la = reshape(Pc,size(Pc,1)*size(Pc,2),1);
  pi = reshape(Pi,size(Pi,1)*size(Pi,2),1);
  p = reshape(Pj,size(Pj,1)*size(Pj,2),1);
  pi = reshape(Pi,size(Pi,1)*size(Pi,2),1);
  lap = reshape(Pcp,size(Pcp,1)*size(Pcp,2),1);
  pip = reshape(Pip,size(Pip,1)*size(Pip,2),1);
  pp = reshape(Pjp,size(Pjp,1)*size(Pjp,2),1);  
% Computes first derivative
% p with respect eta' (D) and of pp with respect eta' (Dp)
  if sw > 1,
    D = [];    
    for s = 1:nst,
      for c = 1:ncl,
        i = sc + (s-1)*ncl*t + (c-1)*t;
        i = i+(1:t);
        v = eta(i); v = glinv(a,M,C,v);
        Dc = inv(C*inv(diag(a+M*v))*M);
        D = blkdiag(D,Pip(c,s)*Dc);
      end
    end
    Dp = kron(eye(nst*ncl),[-ones(1,t); eye(t)])*D;
    Dt = []; Dtp = [];
    for s = 1:nst,
      v = Pi(:,s);
      O = diag(v)-v*v';
      Op = [-sum(O'); O];
      Dt = blkdiag(Dt,diag(Pc(:,s))*kron(eye(ncl),ones(t,1))*Op);
      Dtp = blkdiag(Dtp,diag(Pcp(:,s))*kron(eye(ncl),ones(t+1,1))*Op);
    end   
    D = [Dt D];
    Dp = [Dtp Dp];
  end

