function b = stglom(Y,nst,ncl,lev,Z,a,M,C,rec)

% -      b = stglom(Y,nst,ncl,lev,Z,a,M,C,<rec>)
% - computes random starting values
% -
% - inputs:
% - y    : incomplete data vector
% - lo   : factor levels observed (those on the right run faster)
% - lc   : number of latent classes
% - Z    : design matrix
% - rec  : 1 for capture rectpure problem, 0 otherwise (default)
% - 
% - outputs:
% - b    : starting values

% - preliminaries
  if nargin < 8, rec = 0; end
  if rec == 0,
     yl = size(Y,1); J = length(lev);
     sc = nst*ncl;
     ls = (1:ncl)-(ncl+1)/2;
     Xs = punt(lev);
     R = kron(ones(1,nst),exp((Xs+1)*ls));
     R = R*diag(sum(R).^(-1));
     R = diag(sum(R').^(-1))*R*nst;
     V = kron(Y,ones(1,ncl)).*R;
     v = sum(V)';
     v = kron(eye(nst),ones(1,ncl)) * v;
     v = kron(v,ones(ncl,1));
     P = V*diag(1./v);
     pm = sum(P)';
     eta =[]; % prob di appartenere alle varie clasee
     for s = 1:nst,
        ia = (s-1)*ncl+1;
        ib = ia+(1:ncl-1);
        eta = [eta; log(pm(ib)/pm(ia))];
     end
     Pc = P * diag(1./pm);
     for s = 1:nst,
        for c = 1:ncl;
           i = (s-1)*ncl + c;
           p = Pc(:,i); p(1) = [];
           eta = [eta; C*log(a+M*p)];
        end
     end 
     b = inv(Z'*Z)*Z'*eta;
  else
     yl = size(Y,1); J = length(lev);
     lkmax = -10^10;
     lv = []; rv = []; etav = [];
     for rho = 0.2:0.05:0.5,
        sc = nst*ncl;
        ls = (1:ncl)-(ncl+1)/2;
        Xs = punt(lev);
        Xs = Xs(2:size(Xs,1),:);
        R = kron(ones(1,nst),exp((Xs+1)*ls));
        R = R*diag(sum(R).^(-1));
        R = diag(sum(R').^(-1))*R*nst;
        V = kron(Y,ones(1,ncl)).*R;
        V = [rho*sum(V); V]; 
        v = sum(V)';
        v = kron(eye(nst),ones(1,ncl)) * v;
        v = kron(v,ones(ncl,1));
        P = V*diag(1./v);
        pm = sum(P)';
        eta =[]; % prob di appartenere alle varie clasee
        for s = 1:nst,
           ia = (s-1)*ncl+1;
           ib = ia+(1:ncl-1);
           eta = [eta; log(pm(ib)/pm(ia))];
        end
        Pc = P * diag(1./pm);
        for s = 1:nst,
           for c = 1:ncl;
              i = (s-1)*ncl + c;
              p = Pc(:,i); p(1) = [];
              eta = [eta; C*log(a+M*p)];
           end
        end 
        b = inv(Z'*Z)*Z'*eta; 
        [lk,gv,Fm] = lkyglom(nst,ncl,J,Y,Z,b,a,M,C,rec);
        rv = [rv; rho];
        lv = [lv; lk];
        etav = [etav eta];
        if lk > lkmax, 
           lkmax = lk;
           bmax = b;
        end
        b = bmax + randn(length(bmax),1)/10;
     end
  end
